<?php

namespace Drupal\eca_content\Event;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;

/**
 * Dispatches on event-based options selection.
 *
 * @internal
 *   This class is not meant to be used as a public API. It is subject for name
 *   change or may be removed completely, also on minor version updates.
 */
class OptionsSelection extends FieldSelectionBase {

  /**
   * A stack of selection event instances.
   *
   * An instance will be removed by
   * \Drupal\eca_content\Plugin\ECA\Event\ContentEntityEvent::cleanupAfterSuccessors.
   *
   * @var \Drupal\eca_content\Event\OptionsSelection[]
   */
  public static array $instances = [];

  /**
   * The field storage definition.
   *
   * @var \Drupal\Core\Field\FieldStorageDefinitionInterface
   */
  public FieldStorageDefinitionInterface $fieldStorageDefinition;

  /**
   * The according entity.
   *
   * @var \Drupal\Core\Entity\ContentEntityInterface|null
   */
  public ?ContentEntityInterface $entity;

  /**
   * The current list of allowed values.
   *
   * @var array
   */
  public array $allowedValues;

  /**
   * Constructs a new event.
   *
   * @param \Drupal\Core\Field\FieldStorageDefinitionInterface $field_storage_definition
   *   The field storage definition.
   * @param \Drupal\Core\Entity\ContentEntityInterface|null $entity
   *   The according entity.
   * @param array $allowed_values
   *   The current list of allowed values.
   */
  public function __construct(FieldStorageDefinitionInterface $field_storage_definition, ?ContentEntityInterface $entity, array $allowed_values) {
    $this->fieldStorageDefinition = $field_storage_definition;
    $this->entity = $entity;
    $this->allowedValues = $allowed_values;
  }

  /**
   * {@inheritdoc}
   */
  public function applies(string $id, array $arguments): bool {
    if (!$this->entity) {
      // Can't do anything without an entity and without a specified field.
      return FALSE;
    }

    if (!_eca_content_entity_types()->bundleFieldApplies($this->entity, $arguments['type'])) {
      return FALSE;
    }
    $field_name = $this->fieldStorageDefinition->getName();
    if (!empty($arguments['field_name']) && (trim($arguments['field_name']) !== $field_name)) {
      return FALSE;
    }

    self::$instances[] = $this;
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function appliesForLazyLoadingWildcard(string $wildcard): bool {
    if (!$this->entity) {
      // Can't do anything without an entity and without a specified field.
      return FALSE;
    }

    $field_name = $this->fieldStorageDefinition->getName();
    $candidates = ['*::*::*'];
    $candidates[] = '*::*::' . trim($field_name);
    $candidates[] = $this->entity->getEntityTypeId() . '::*::*';
    $candidates[] = $this->entity->getEntityTypeId() . '::' . $this->entity->bundle() . '::*';
    $candidates[] = $this->entity->getEntityTypeId() . '::*::' . trim($field_name);
    $candidates[] = $this->entity->getEntityTypeId() . '::' . $this->entity->bundle() . '::' . trim($field_name);
    return in_array($wildcard, $candidates);
  }

  /**
   * {@inheritdoc}
   */
  public function getEntity(): ContentEntityInterface {
    return $this->entity;
  }

}
