<?php

namespace Drupal\eca_form\Event;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\eca\Event\ContentEntityEventInterface;
use Drupal\eca\Event\RenderEventInterface;
use Drupal\eca\Plugin\DataType\DataTransferObject;
use Drupal\eca\Token\DataProviderInterface;
use Drupal\eca\Token\TokenServices;

/**
 * Dispatched when an inline entity form is being build.
 *
 * @internal
 *   This class is not meant to be used as a public API. It is subject for name
 *   change or may be removed completely, also on minor version updates.
 *
 * @package Drupal\eca_form\Event
 */
class InlineEntityFormBuild extends FormBase implements RenderEventInterface, ContentEntityEventInterface, DataProviderInterface {

  /**
   * The embedded entity that belongs to the inline entity form.
   *
   * @var \Drupal\Core\Entity\ContentEntityInterface
   */
  protected ContentEntityInterface $entity;

  /**
   * The parent entity.
   *
   * @var \Drupal\Core\Entity\ContentEntityInterface
   */
  protected ContentEntityInterface $parent;

  /**
   * The name of the field (of the parent) that renders the inline entity form.
   *
   * @var string
   */
  protected string $fieldName;

  /**
   * The delta of the entity reference in the field.
   *
   * @var int
   */
  protected int $delta;

  /**
   * The widget plugin ID used to render the inline entity form.
   *
   * @var string
   */
  protected string $widgetPluginId;

  /**
   * An instance holding event data accessible as Token.
   *
   * @var \Drupal\eca\Plugin\DataType\DataTransferObject|null
   */
  protected ?DataTransferObject $eventData = NULL;

  /**
   * Constructs an InlineEntityFormBuild instance.
   *
   * @param array &$subform
   *   The render array of the inline entity form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The parent form state.
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity.
   * @param \Drupal\Core\Entity\ContentEntityInterface $parent
   *   The parent.
   * @param string $field_name
   *   The field name of the parent.
   * @param int $delta
   *   The delta within the field of the parent.
   * @param string $widget_plugin_id
   *   The plugin ID of the field widget.
   */
  public function __construct(array &$subform, FormStateInterface $form_state, ContentEntityInterface $entity, ContentEntityInterface $parent, string $field_name, int $delta, string $widget_plugin_id) {
    parent::__construct($subform, $form_state);
    $this->entity = $entity;
    $this->parent = $parent;
    $this->fieldName = $field_name;
    $this->delta = $delta;
    $this->widgetPluginId = $widget_plugin_id;
  }

  /**
   * {@inheritdoc}
   */
  public function &getRenderArray(): array {
    $form = &$this->getForm();
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getEntity(): ContentEntityInterface {
    return $this->entity;
  }

  /**
   * Get the parent entity.
   *
   * @return \Drupal\Core\Entity\ContentEntityInterface
   *   The parent entity.
   */
  public function getParent(): ContentEntityInterface {
    return $this->parent;
  }

  /**
   * Get field name of the parent that renders the inline entity form.
   *
   * @return string
   *   The field name.
   */
  public function getFieldName(): string {
    return $this->fieldName;
  }

  /**
   * Get the delta of the entity reference in the field.
   *
   * @return int
   *   The delta.
   */
  public function getDelta(): int {
    return $this->delta;
  }

  /**
   * Get the widget plugin ID.
   *
   * @return string
   *   The widget plugin ID.
   */
  public function getWidgetPluginId(): string {
    return $this->widgetPluginId;
  }

  /**
   * {@inheritdoc}
   */
  public function getData(string $key): mixed {
    $entity = $this->getEntity();

    if ($key === 'event') {
      if (!isset($this->eventData)) {
        $form_object = $this->formState->getFormObject();
        $parent = $form_object instanceof EntityFormInterface ? $form_object->getEntity() : NULL;
        $data = [
          'machine-name' => FormEvents::IEF_BUILD,
          'entity' => $entity,
          'entity-type' => $entity->getEntityTypeId(),
          'entity-bundle' => $entity->bundle(),
          'parent' => $parent,
          'field-name' => $this->getFieldName(),
          'widget' => $this->getWidgetPluginId(),
          'delta' => $this->getDelta(),
        ];
        if (!$entity->isNew()) {
          $data['entity-id'] = $entity->id();
        }
        if ($parent) {
          $data['parent-type'] = $parent->getEntityTypeId();
          $data['parent-bundle'] = $parent->bundle();
          if (!$parent->isNew()) {
            $data['parent-id'] = $parent->id();
          }
        }
        $this->eventData = DataTransferObject::create($data);
      }

      return $this->eventData;
    }

    if ($key === 'entity') {
      return $entity;
    }

    if ($key === TokenServices::get()->getTokenType($entity)) {
      return $entity;
    }

    if ($key === $entity->getEntityTypeId()) {
      return $this->getEntity();
    }

    if ($key === 'parent') {
      $form_object = $this->formState->getFormObject();
      $parent = $form_object instanceof EntityFormInterface ? $form_object->getEntity() : NULL;
      return $parent;
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function hasData(string $key): bool {
    return $this->getData($key) !== NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function appliesForLazyLoadingWildcard(string $wildcard): bool {
    [
      $w_form_ids,
      $w_entity_type_ids,
      $w_bundles,
      $w_operations,
      $w_parent_type_ids,
      $w_parent_bundles,
      $w_field_names,
    ] = explode(':', $wildcard);

    // Check for the specified parent type, if any.
    if (!parent::appliesForLazyLoadingWildcard(implode(':', [
      $w_form_ids,
      $w_parent_type_ids,
      $w_parent_bundles,
      $w_operations,
    ]))) {
      return FALSE;
    }

    // Check for the embedded entity type.
    $entity = $this->getEntity();
    if (($w_entity_type_ids !== '*') && !in_array($entity->getEntityTypeId(), explode(',', $w_entity_type_ids), TRUE)) {
      return FALSE;
    }
    if (($w_bundles !== '*') && !in_array($entity->bundle(), explode(',', $w_bundles), TRUE)) {
      return FALSE;
    }
    if (($w_field_names !== '*') && !in_array($this->getFieldName(), explode(',', $w_field_names), TRUE)) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function applies(string $id, array $arguments): bool {
    if (!$this->appliesForEntityTypeOrBundle($this->getEntity(), $arguments)) {
      return FALSE;
    }

    $parent_arguments = [
      'entity_type_id' => $arguments['parent_type_id'] ?? '*',
      'bundle' => $arguments['parent_bundle'] ?? '*',
    ] + $arguments;
    if (!parent::applies($id, $parent_arguments)) {
      return FALSE;
    }

    if (!empty($arguments['field_name']) && $arguments['field_name'] !== '*') {
      if (!in_array($this->getFieldName(), array_map('strtolower', array_map('trim', explode(',', $arguments['field_name']))), TRUE)) {
        return FALSE;
      }
    }

    return TRUE;
  }

}
